@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Find Exif data in an HEIC file.
 
 # As of 2019, the latest standard seems to be "ISO/IEC 14496-12:2015"
@@ -13,11 +11,13 @@
 #      gives us position and size information.
 
 import struct
+from typing import List, Dict, Callable, BinaryIO, Optional
 
-from .exif_log import get_logger
+from exifread.exif_log import get_logger
 
 logger = get_logger()
 
+
 class WrongBox(Exception):
     pass
 class NoParser(Exception):
@@ -28,29 +28,62 @@ class BadSize(Exception):
     pass
 
 
-class Box:  # pylint: disable=too-few-public-methods
-    def __init__(self, name):
-        self.name = name
-        self.size = None
-        self.after = None
-        self.pos = None
-        self.item_id = None
+class Box:
+    version = 0
+    minor_version = 0
+    item_count = 0
+    size = 0
+    after = 0
+    pos = 0
+    compat = []  # type: List
+    base_offset = 0
+    # this is full of boxes, but not in a predictable order.
+    subs = {}  # type: Dict[str, Box]
+    locs = {}  # type: Dict
+    exif_infe = None  # type: Optional[Box]
+    item_id = 0
+    item_type = b''
+    item_name = b''
+    item_protection_index = 0
+    major_brand = b''
+    offset_size = 0
+    length_size = 0
+    base_offset_size = 0
+    index_size = 0
+    flags = 0
 
-    def __repr__(self):
+    def __init__(self, name: str):
+        self.name = name
+
+    def __repr__(self) -> str:
         return "<box '%s'>" % self.name
 
+    def set_sizes(self, offset: int, length: int, base_offset: int, index: int):
+        self.offset_size = offset
+        self.length_size = length
+        self.base_offset_size = base_offset
+        self.index_size = index
+
+    def set_full(self, vflags: int):
+        """
+        ISO boxes come in 'old' and 'full' variants.
+        The 'full' variant contains version and flags information.
+        """
+        self.version = vflags >> 24
+        self.flags = vflags & 0x00ffffff
+
 
 class HEICExifFinder:
 
-    def __init__(self, file_handle):
+    def __init__(self, file_handle: BinaryIO):
         self.file_handle = file_handle
 
-    def get(self, nbytes):
+    def get(self, nbytes: int) -> bytes:
         read = self.file_handle.read(nbytes)
         if not read:
             raise EOFError
         if len(read) != nbytes:
-            msg = "get(nbytes={nbytes}) found {read} bytes at postion {pos}".format(
+            msg = "get(nbytes={nbytes}) found {read} bytes at position {pos}".format(
                 nbytes=nbytes,
                 read=len(read),
                 pos=self.file_handle.tell()
@@ -58,23 +91,23 @@ class HEICExifFinder:
             raise BadSize(msg)
         return read
 
-    def get16(self):
+    def get16(self) -> int:
         return struct.unpack('>H', self.get(2))[0]
 
-    def get32(self):
+    def get32(self) -> int:
         return struct.unpack('>L', self.get(4))[0]
 
-    def get64(self):
+    def get64(self) -> int:
         return struct.unpack('>Q', self.get(8))[0]
 
-    def get_int4x2(self):
+    def get_int4x2(self) -> tuple:
         num = struct.unpack('>B', self.get(1))[0]
         num0 = num >> 4
         num1 = num & 0xf
         return num0, num1
 
-    # some fields have variant-sized data.
-    def get_int(self, size):
+    def get_int(self, size: int) -> int:
+        """some fields have variant-sized data."""
         if size == 2:
             return self.get16()
         if size == 4:
@@ -85,7 +118,7 @@ class HEICExifFinder:
             return 0
         raise BadSize(size)
 
-    def get_string(self):
+    def get_string(self) -> bytes:
         read = []
         while 1:
             char = self.get(1)
@@ -94,7 +127,7 @@ class HEICExifFinder:
             read.append(char)
         return b''.join(read)
 
-    def next_box(self):
+    def next_box(self) -> Box:
         pos = self.file_handle.tell()
         size = self.get32()
         kind = self.get(4).decode('ascii')
@@ -113,37 +146,40 @@ class HEICExifFinder:
         box.pos = self.file_handle.tell()
         return box
 
-    def get_full(self, box):
-        # iso boxes come in 'old' and 'full' variants.  the 'full' variant
-        #   contains version and flags information.
-        vflags = self.get32()
-        box.version = vflags >> 24
-        box.flags = vflags & 0x00ffffff
+    def get_full(self, box: Box):
+        box.set_full(self.get32())
 
-    def skip(self, box):
+    def skip(self, box: Box):
         self.file_handle.seek(box.after)
 
-    def expect_parse(self, name):
+    def expect_parse(self, name: str) -> Box:
         while True:
             box = self.next_box()
             if box.name == name:
                 return self.parse_box(box)
             self.skip(box)
 
-    def get_parser(self, box):
-        method = 'parse_%s' % box.name
-        return getattr(self, method, None)
+    def get_parser(self, box: Box) -> Callable:
+        defs = {
+            'ftyp': self._parse_ftyp,
+            'meta': self._parse_meta,
+            'infe': self._parse_infe,
+            'iinf': self._parse_iinf,
+            'iloc': self._parse_iloc,
+        }
+        try:
+            return defs[box.name]
+        except (IndexError, KeyError) as err:
+            raise NoParser(box.name) from err
 
-    def parse_box(self, box):
+    def parse_box(self, box: Box) -> Box:
         probe = self.get_parser(box)
-        if probe is None:
-            raise NoParser(box.name)
         probe(box)
         # in case anything is left unread
         self.file_handle.seek(box.after)
         return box
 
-    def parse_ftyp(self, box):
+    def _parse_ftyp(self, box: Box):
         box.major_brand = self.get(4)
         box.minor_version = self.get32()
         box.compat = []
@@ -152,10 +188,8 @@ class HEICExifFinder:
             box.compat.append(self.get(4))
             size -= 4
 
-    def parse_meta(self, meta):
+    def _parse_meta(self, meta: Box):
         self.get_full(meta)
-        # this is full of boxes, but not in a predictable order.
-        meta.subs = {}
         while self.file_handle.tell() < meta.after:
             box = self.next_box()
             psub = self.get_parser(box)
@@ -167,7 +201,7 @@ class HEICExifFinder:
             # skip any unparsed data
             self.skip(box)
 
-    def parse_infe(self, box):
+    def _parse_infe(self, box: Box):
         self.get_full(box)
         if box.version >= 2:
             if box.version == 2:
@@ -178,10 +212,8 @@ class HEICExifFinder:
             box.item_type = self.get(4)
             box.item_name = self.get_string()
             # ignore the rest
-        else:
-            box.item_type = ''
 
-    def parse_iinf(self, box):
+    def _parse_iinf(self, box: Box):
         self.get_full(box)
         count = self.get16()
         box.exif_infe = None
@@ -192,14 +224,11 @@ class HEICExifFinder:
                 box.exif_infe = infe
                 break
 
-    def parse_iloc(self, box):
+    def _parse_iloc(self, box: Box):
         self.get_full(box)
         size0, size1 = self.get_int4x2()
         size2, size3 = self.get_int4x2()
-        box.offset_size = size0
-        box.length_size = size1
-        box.base_offset_size = size2
-        box.index_size = size3
+        box.set_sizes(size0, size1, size2, size3)
         if box.version < 2:
             box.item_count = self.get16()
         elif box.version == 2:
@@ -232,11 +261,12 @@ class HEICExifFinder:
                 extents.append((extent_offset, extent_length))
             box.locs[item_id] = extents
 
-    def find_exif(self):
+    def find_exif(self) -> tuple:
         ftyp = self.expect_parse('ftyp')
         assert ftyp.major_brand == b'heic'
         assert ftyp.minor_version == 0
         meta = self.expect_parse('meta')
+        assert meta.subs['iinf'].exif_infe is not None
         item_id = meta.subs['iinf'].exif_infe.item_id
         extents = meta.subs['iloc'].locs[item_id]
         logger.debug('HEIC: found Exif location.')
